@@ -18,7 +18,7 @@ check <- function(object, object.name, check.name, check.func, na.ok=FALSE)
         which.na <- which(is.na(any))
         if(length(which.na)) {
             stopf("NA in %s\n       %s[%d] is %g",
-                  object.name, unquote(object.name),
+                  object.name, object.name,
                   which.na[1], object[which.na[1]])
         }
     }
@@ -26,7 +26,7 @@ check <- function(object, object.name, check.name, check.func, na.ok=FALSE)
         which <- which(check.func(object))
         stopifnot(length(which) > 0)
         stopf("%s in %s\n       %s[%d] is %g",
-              check.name, object.name, unquote(object.name),
+              check.name, object.name, object.name,
               which[1], object[which[1]])
     }
 }
@@ -52,8 +52,8 @@ check <- function(object, object.name, check.name, check.func, na.ok=FALSE)
 #             printf("\n")
 #             s <- paste0(strwrap(list.as.char(args),
 #                         width=getOption("width"), exdent=7), collapse="\n")
-#             stop0("cannot evaluate '", names(args)[i],
-#                   "' in\n       ", fname, "(", s, ")")
+#             stop0("cannot evaluate ", quotify(names(args)[i], "'"),
+#                   " in\n       ", fname, "(", s, ")")
 #         }
 #     }
 # }
@@ -67,7 +67,7 @@ check.boolean <- function(b) # b==0 or b==1 is also ok
         stop0("the ", short.deparse(substitute(b), "given"),
               " argument is not FALSE, TRUE, 0, or 1")
     if(!(is.logical(b) || is.numeric(b)) || is.na(b) || !(b == 0 || b == 1))
-        stop0(short.deparse(substitute(b), "the argument"), "=", b,
+        stop0(short.deparse(substitute(b), "the argument"), "=", as.char(b),
             " but it should be FALSE, TRUE, 0, or 1")
     b != 0 # convert to logical
 }
@@ -79,40 +79,52 @@ is.boolean <- function(b) # b==NA or b==0 or b==1
 }
 check.classname <- function(object, substituted.object, allowed.classnames)
 {
-    err.msg <- quotify(allowed.classnames)
+    expected.classname <- quotify(allowed.classnames)
     if(length(allowed.classnames) > 1)
-        err.msg <- sprint("one of\n%s", err.msg)
+        expected.classname <- sprint("one of\n%s", expected.classname)
     if(is.null(object))
         stopf("object is NULL but expected an object of class of %s",
-              err.msg)
+              expected.classname)
     if(!inherits(object, allowed.classnames)) {
-        stopf("the class of '%s' is \"%s\" but expected the class to be %s",
-              paste.trunc(substituted.object, maxlen=30),
-              class(object)[1], err.msg)
+        stopf("the class of %s is \"%s\" but expected the class to be %s",
+              quotify(paste.trunc(substituted.object, maxlen=30)),
+              class(object)[1], expected.classname)
     }
 }
+# adjust name so e.g. error message is "argument is NULL" not "NULL is NULL"
+tweak.name <- function(name, quote=TRUE)
+{
+    quoted.name <- quotify(name, quote="'")
+    if(name %in% c("NULL", "NA") ||
+           (substr(name[1], 1, 1) %in% c("+", "-")) ||
+           grepl("[0-9]", substr(name[1], 1, 1))) {
+        quoted.name <- name <- "argument"
+    }
+    if(quote) quoted.name else name
+}
 check.integer.scalar <- function(object, min=NA, max=NA, null.ok=FALSE,
                                  na.ok=FALSE, logical.ok=TRUE,
                                  char.ok=FALSE,
                                  object.name=short.deparse(substitute(object)))
 {
-    stop.msg <- function()
+    stop.msg <- function(s)
     {
         s.null    <- if(null.ok)    ", or NULL"          else ""
         s.na      <- if(na.ok)      ", or NA"            else ""
         s.logical <- if(logical.ok) ", or TRUE or FALSE" else ""
         s.char    <- if(char.ok)    ", or a string"      else ""
-        stop0(object.name, "=", object[1], " but it should be an an integer",
+        stop0(s,
+              " but it should be an integer",
               s.null, s.na, s.logical, s.char)
     }
     if(is.character(object)) {
         if(!char.ok || length(object) != 1)
-            stop.msg()
+            stop.msg(paste0(tweak.name(object.name), " is a string"))
     } else {
         check.numeric.scalar(object, min, max, null.ok, na.ok, logical.ok,
                              char.ok.msg=char.ok, object.name=object.name)
         if(!is.null(object) && !is.na(object) && object != floor(object))
-                stop.msg()
+            stop.msg(paste0(tweak.name(object.name, quote=FALSE), "=", object[1]))
     }
     object
 }
@@ -139,7 +151,7 @@ check.no.na.in.mat <- function(object)
                     sprint("%s[,%d]",
                         short.deparse(substitute(object), "matrix"), icol)
 
-            check(object, check.name, "NA", is.na, na.ok=FALSE)
+            check(object[,icol], check.name, "NA", is.na, na.ok=FALSE)
         }
     }
 }
@@ -167,32 +179,38 @@ check.numeric.scalar <- function(object, min=NA, max=NA, null.ok=FALSE,
     s.logical <- if(logical.ok) ", or TRUE or FALSE" else ""
     if(na.ok)
         logical.ok <- TRUE # needed because NA is a logical
-    any.na <- !is.null(object) && anyNA(object)
+    any.na <- !is.null(object) &&
+                # following needed because anyNA gives error on some objects
+                (is.numeric(object) || is.logical(object) ||
+                    is.list(object) || is.character(object)) &&
+                anyNA(object)
     if(is.null(object)) {
         if(!null.ok)
-            stop0(object.name, "=NULL is not allowed")
+            stop0(tweak.name(object.name), " is NULL")
     } else if(any.na && !na.ok)
-        stop0(object.name, "=NA is not allowed")
+        stop0(tweak.name(object.name), " is NA")
     else if(!is.numeric(object) && !(is.logical(object) && logical.ok)) {
         s.na   <- if(na.ok)       ", or NA"       else ""
         s.null <- if(null.ok)     ", or NULL"     else ""
         s.char <- if(char.ok.msg) ", or a string" else ""
-        stopf("'%s' must be numeric%s%s%s%s (whereas its current class is \"%s\")",
-              object.name, s.null, s.na, s.char, s.logical, class(object)[1])
+        stopf("%s must be numeric%s%s%s%s (whereas its current class is %s)",
+              tweak.name(object.name),
+              s.null, s.na, s.char, s.logical,
+              class.as.char(object, quotify=TRUE))
     } else if(length(object) != 1)
-        stopf("the length of '%s' must be 1 (whereas its current length is %d)",
-              object.name, length(object))
-    if(!is.null(object) && !is.na(object)) {
+        stopf("the length of %s must be 1 (whereas its current length is %d)",
+              tweak.name(object.name), length(object))
+    if(!is.null(object) && !any.na) {
         if(!is.na(min) && !is.na(max) && (object < min || object > max)) {
-            stop0(object.name, "=", object,
+            stop0(tweak.name(object.name, quote=FALSE), "=", object,
                   " but it should be between ", min, " and ", max)
         }
         if(!is.na(min) && object < min) {
-            stop0(object.name, "=", object,
+            stop0(tweak.name(object.name, quote=FALSE), "=", object,
                   " but it should be at least ", min)
         }
         if(!is.na(max) && object > max) {
-            stop0(object.name, "=", object,
+            stop0(tweak.name(object.name, quote=FALSE), "=", object,
                   " but it should not be greater than ", max)
         }
     }
@@ -221,12 +239,12 @@ check.that.most.are.positive <- function(x, xname, user.arg, non.positive.msg, f
 check.vec <- function(object, object.name, expected.len=NA, logical.ok=TRUE, na.ok=FALSE)
 {
     if(!(NROW(object) == 1 || NCOL(object) == 1))
-        stop0(object.name, " is not a vector\n       ",
+        stop0(tweak.name(object.name), " is not a vector\n       ",
               "It has dimensions ", NROW(object), " by ", NCOL(object))
     if(!((logical.ok && is.logical(object)) || is.numeric(object)))
-        stop0(object.name, " is not numeric")
+        stop0(tweak.name(object.name), " is not numeric")
     if(!is.na(expected.len) && length(object) != expected.len)
-        stop0(object.name, " has the wrong length ",
+        stop0(tweak.name(object.name), " has the wrong length ",
               length(object), ", expected ", expected.len)
     if(na.ok)
         object[is.na(object)] <- 1 # prevent check is.finite from complaining
@@ -234,6 +252,15 @@ check.vec <- function(object, object.name, expected.len=NA, logical.ok=TRUE, na.
         check(object, object.name, "NA", is.na)
     check(object, object.name, "non-finite value", function(object) {!is.finite(object)})
 }
+cleantry <- function(err) # clean up a try.err (remove "Error: " etc.)
+{
+    stopifnot(is.try.err(err))
+    attributes(err) <- NULL
+    err <- gsub("^[^:]*: *", "", err) # remove "Error: " (actually everything up to the first colon)
+    err <- gsub("\n", " ", err, fixed=TRUE)  # remove newlines
+    err <- gsub("  +", " ", err)             # multiple spaces to single spaces
+    gsub(" $", "", err)                      # remove trailing space
+}
 # returns the column name, if that is not possible then something like x[,1]
 colname <- function(object, i, object.name=trunc.deparse(substitute(object)))
 {
@@ -308,26 +335,27 @@ exp10 <- function(x) # e.g. exp10(-3) = 1e-3
 
 fix.lim <- function(lim)
 {
-    if(!is.null(lim) && !isDate(lim)) {
+    if(!is.null(lim) && !inherits(lim, "Date")) {
         stopifnot(is.numeric(lim), length(lim) == 2)
         # constants below are arbitrary
-        small <- max(1e-6, .001 * abs(lim[1]), .001 * abs(lim[2]))
+        small <- max(1e-6, .01 * abs(lim[1] - lim[2]))
         if(abs(lim[2] - lim[1]) < small) # illegal lim?
             lim <- c(lim[1] - small, lim[2] + small)
     }
     lim
 }
-# Ensure all columns of x have column names.  Won't overwrite existing
-# column names (TODO except possibly if make.unique kicks in).
+# Ensure all columns of x have column names.  Won't overwrite existing column names.
 
-gen.colnames <- function(x, prefix="x", alt.prefix=prefix, trace=0, xname=NULL)
+gen.colnames <- function(x, prefix="x", alt.prefix=prefix, trace=0)
 {
     if(NCOL(x) == 0)
         return(NULL)
+
     # If prefix is long and has characters like ( or [ then use the
     # alternate prefix.  This is sometimes necessary when prefix is
     # generated using deparse and the arg is something like
     # "cbind(trees$Volume,trees$Volume+100)"
+
     if(any(nchar(prefix) > 30) && grepany("[([,]", prefix)) {
         trace2(trace, "using alt.prefix \"%s\" instead of prefix \"%s\"\n",
                alt.prefix, prefix)
@@ -350,10 +378,12 @@ gen.colnames <- function(x, prefix="x", alt.prefix=prefix, trace=0, xname=NULL)
         if(any(missing))
             colnames[missing] <- new.colnames[missing]
     }
-    colnames <- make.unique(strip.space(colnames))
+    if(length(unique(colnames)) != length(colnames))
+        stop0("Duplicate colname in ", paste.trunc(prefix),
+              " (colnames are ",
+              paste.with.quotes(colnames, maxlen=60), ")")
     if(trace >= 2 && !identical(org.colnames, colnames))
-        trace2(trace, "%s colname%s %s now %s\n",
-            if(is.null(xname)) trunc.deparse(substitute(x)) else xname,
+        trace2(trace, "colname%s %s now %s\n",
             if(length(colnames) > 1) "s were" else " was",
             if(is.null(org.colnames)) "NULL"
             else paste.trunc(quotify(org.colnames)),
@@ -377,39 +407,69 @@ get.mean.rsq <- function(rss, tss, wp)
 #   3. Else return the environment in which the caller of this function
 #      was called (e.g. return the environment of plotmo's caller).
 
-get.model.env <- function(object, object.name="object", trace=0)
+get.model.env <- function(object, object.name="object", trace=0, use.submodel=FALSE)
 {
     # check args, because this func is called very early in plotmo (and friends)
-    stopifnot.string(object.name)
     check.numeric.scalar(trace, logical.ok=TRUE)
-    if(is.null(object))
-        stopf("%s is NULL", object.name)
-    if(!is.list(object))
-        stopf("%s is not an S3 model", object.name)
-    if(class(object)[1] == "list")
-        stopf("%s is a plain list, not an S3 model", object.name)
     if(trace >= 2) {
         callers.name <- callers.name()
         my.call <- call.as.char(n=2)
         printf.wrap("%s trace %g: %s\n", callers.name, trace, my.call)
-        call <- getCall(object)
-        if(is.null(call))
-            printf("object class is \"%s\" with no call\n", class(object)[1])
-        else
-            printf.wrap("object call is %s\n", strip.deparse(call))
-        printf("--get.model.env for %s object\n", class(object)[1])
+        printf("--get.model.env for object with class %s\n", class.as.char(object))
     }
-    # following will fail for non-formula models because they have no terms field
-    terms <- try(terms(object), silent=trace < 3)
-    if(!is.try.err(terms) && !is.null(terms)) {
+    stopifnot.string(object.name)
+    if(is.null(object))
+        stopf("argument %s is NULL", object.name)
+    if(!is.list(object))
+        stopf("%s is not an S3 model", object.name)
+    if(class(object)[1] == "list") # some packages build models without a specific class
+        stopf("%s is a plain list, not an S3 model", object.name)
+    obj <- object
+
+    # Special handling for parsnip models. Their class is like c("_earth", "model_fit").
+    # For these models, use the env if any saved with the submod (e.g. earth)
+    # (We don't do this for caret models because caret models have a terms field.)
+    #
+    # TODO this code is preliminary (works with parsnip 0.1.3)
+    # and only works if model saves the data (e.g. lm, earth(keepxy=TRUE), not rpart
+    if(use.submodel && inherits(object, "model_fit")) { # parsnip
+        trace2(trace, "plotmo parsnip model: will plot %s$fit, not %s itself\n",
+               gsub("'", "", object.name), object.name)
+        obj <- object[["fit"]]
+        if(!is.list(obj)) # sanity check
+            stopf("plotmo parsnip model: %s$fit is not an S3 model",
+                  gsub("'", "", object.name))
+        # TODO following is temporary, hopefully
+        if(inherits(obj, "rpart") && is.null(obj$model))
+            stop0(
+                "Cannot plot parsnip rpart model: need model=TRUE in call to rpart\n",
+                "       Do it like this: set_engine(\"rpart\", model=TRUE)")
+    }
+    if(trace >= 2) {
+        call <- getCall(obj)
+        if(is.null(call))
+            printf("object has no call field (it's class is %s)\n", class.as.char(object))
+        else
+            printf.wrap("object call is %s\n", strip.deparse(call), maxlen=120)
+    }
+    terms <- try(terms(obj), silent=trace < 3)
+
+    # Following will fail (correctly) for non-formula models because they have no terms.
+    #
+    # TODO Also, if use.submodel, don't use terms (because the term env was
+    #      inside the parsnip func that created the submodel)
+    #      But that also fails later when we eval the formula because
+    #      eval will use GlobalEnv instead of the data passed to the model
+
+    if(!is.null(terms) && !is.try.err(terms)) {
         model.env <- attr(terms, ".Environment")
         if(is.null(model.env)) {
-            if(inherits(object, "glmnet.formula") ||  # glmnetUtils package
-               inherits(object, "cv.glmnet.formula"))
-                if(inherits(object, "glmnet.formula"))
+            if(inherits(obj, "glmnet.formula") ||  # glmnetUtils package
+               inherits(obj, "cv.glmnet.formula"))
+                if(inherits(obj, "glmnet.formula"))
                     stop0(
 "for this plot, glmnet.formula must be called with use.model.frame=TRUE")
-                if(inherits(object, "cv.glmnet.formula"))
+                if(inherits(obj, "cv.glmnet.formula"))
                     stop0(
 "for this plot, cv.glmnet.formula must be called with use.model.frame=TRUE")
             stop0("attr(terms, \".Environment\") is NULL")
@@ -417,25 +477,29 @@ get.model.env <- function(object, object.name="object", trace=0)
         if(!is.environment(model.env))
             stop0("attr(terms, \".Environment\") is not an environment")
         else {
-            trace2(trace, "using the environment saved with the %s model: %s\n",
-                   class(object)[1], environment.as.char(model.env))
+            trace2(trace, "using the environment saved in $terms of the %s model: %s\n",
+                   class.as.char(obj), environment.as.char(model.env))
             return(model.env)
         }
     }
-    model.env <- attr(object, ".Environment")
+    model.env <- attr(obj, ".Environment")
     if(is.environment(model.env)) {
-        trace2(trace, "using attr(object,\".Environment\") saved with %s model: %s\n",
-               class(object)[1], environment.as.char(model.env))
+        trace2(trace, "using attr(obj,\".Environment\") saved with %s model: %s\n",
+               class.as.char(obj), environment.as.char(model.env))
         return(model.env)
     }
     if(!is.null(model.env))
-        stop0("attr(object, \".Environment\") is not an environment")
+        stop0("attr(obj, \".Environment\") is not an environment")
+
+    # n=2 is the caller of the function that called get.model.env
+    # for plotmo it will be the caller of plotmo, typically R_GlobalEnv
+    model.env <- parent.frame(n=2)
 
-    model.env <- parent.frame(n=2) # caller of the function that called model.env
     trace2(trace,
            "assuming the environment of the %s model is that of %s's caller: %s\n",
-           class(object)[1], callers.name, environment.as.char(model.env))
-    return(model.env)
+           class.as.char(obj), callers.name, environment.as.char(model.env))
+
+    model.env
 }
 get.rsq <- function(rss, tss)
 {
@@ -485,17 +549,19 @@ ife <- function(ife.test, ife.yes, ife.no)
 # returns an index, choices is a vector of strings
 imatch.choices <- function(arg, choices,
             argname=short.deparse(substitute(arg), "function"),
-            err.msg.has.index=FALSE, # TRUE if integer "arg" is legal elsewhere
-            err.msg="",              # error message, "" for automatic
-            err.msg.ext="")          # extension to error message
+            errmsg.has.index=FALSE, # TRUE if integer "arg" is legal elsewhere
+            errmsg="",              # error message, "" for automatic
+            errmsg.ext="")          # extension to error message
 {
-    err.msg.ext <- paste0(
-               if(err.msg.has.index) " an integer index or" else "",
-               if(nchar(err.msg.ext)) paste0(" ", err.msg.ext, " or") else "")
-    if(nchar(err.msg) == 0)
-        err.msg <- sprint("Choose%s one of: %s", err.msg.ext, quotify(choices))
+    errmsg.ext <- paste0(
+               if(errmsg.has.index) " an integer index or" else "",
+               if(nchar(errmsg.ext)) paste0(" ", errmsg.ext, " or") else "")
+    if(nchar(errmsg) == 0)
+        errmsg <- sprint("Choose%s one of: %s", errmsg.ext, quotify(choices))
     if(!is.character(arg) || length(arg) != 1 || !nzchar(arg))
-         stopf("illegal '%s' argument\n%s", argname, err.msg)
+         stopf("illegal %s argument\n%s", quotify(argname, "'"), errmsg)
+    if(argname %in% c("NULL", "NA"))
+        argname <- "argument"
     imatch <- pmatch(arg, choices)
     if(anyNA(imatch)) {
         imatch <- NULL
@@ -505,21 +571,17 @@ imatch.choices <- function(arg, choices,
         if(length(imatch) == 0) {
             if(length(choices) == 1)
                 stopf("%s=\"%s\" is not allowed\n       Only%s %s is allowed",
-                      argname, paste(arg), err.msg.ext, quotify(choices))
+                      argname, paste(arg), errmsg.ext, quotify(choices))
             else
                 stopf("%s=\"%s\" is not allowed\n%s",
-                      argname, paste(arg), err.msg)
+                      argname, paste(arg), errmsg)
         }
         if(length(imatch) > 1)
             stopf("%s=\"%s\" is ambiguous\n%s",
-                  argname, paste(arg), err.msg)
+                  argname, paste(arg), errmsg)
     }
     imatch
 }
-isDate <- function(x)
-{
-    inherits(x, "Date")
-}
 # TRUE if all values in object are integers, ignoring NAs
 # assumes object is numeric or logical (check this before call this function)
 is.integral <- function(object)
@@ -578,7 +640,7 @@ lighten <- function(col, lighten.amount, alpha=1)
     else
         rgb(rgb[1,], rgb[2,], rgb[3,], alpha)
 }
-# returns the expanded arg
+# returns the expanded arg (error msg if arg is not an allowed choice in calling func)
 match.arg1 <- function(arg, argname=deparse(substitute(arg)))
 {
     formal.args <- formals(sys.function(sys.parent()))
@@ -586,14 +648,15 @@ match.arg1 <- function(arg, argname=deparse(substitute(arg)))
     formal.argnames[imatch.choices(arg[1], formal.argnames, argname)]
 }
 # returns a string, choices is a vector of strings
+# error msg if arg is not an allowed choice
 match.choices <- function(arg,
             choices,
             argname=deparse(substitute(arg)),
-            err.msg="",              # error message, "" for automatic
-            err.msg.ext="")          # extension to error message
+            errmsg="",              # error message ("" for automatic)
+            errmsg.ext="")          # extension to error message
 {
     choices[imatch.choices(arg, choices, argname,
-                           err.msg=err.msg, err.msg.ext=err.msg.ext)]
+                           errmsg=errmsg, errmsg.ext=errmsg.ext)]
 }
 # This uses the object's .Environment attribute, which was
 # pre-assigned to the object via get.model.env
@@ -632,8 +695,8 @@ my.data.frame <- function(x, trace, stringsAsFactors=TRUE)
         # come here for sparse matrices from the Matrix package
         df <- try(as.matrix(x))
         if(is.try.err(df))
-            stopf("Cannot convert '%s' object to a data.frame or matrix",
-                  class(x)[1])
+            stopf("Cannot convert %s object to a data.frame or matrix",
+                  quotify(class(x)[1]))
         df <- as.data.frame(df, stringsAsFactors=stringsAsFactors)
         trace2(trace, "converted %s object to data.frame\n", class(x)[1])
     }
@@ -665,13 +728,22 @@ nlines <- function(s)
     else
         length(strsplit(s, "\n")[[1]])
 }
-paste.c <- function(object, maxlen=16) # return "x1" or "c(x1, x2)"
+paste.c <- function(object, maxlen=16) # return 'x1' or 'c(x1, x2)'
 {
     if(length(object) == 1)
         paste.trunc(object)
     else
         paste0("c(", paste.trunc(object, collapse=",", maxlen=maxlen), ")")
 }
+paste.with.quotes <- function(object, maxlen=16) # return '"x1"' or '"x1", "x2"'
+{
+    if(is.null(object[1]))
+        "NULL"
+    else if(length(object) == 0)
+        "EMPTY"
+    else
+        paste0(paste.trunc("\"", object, "\"", collapse=", ", sep="", maxlen=maxlen))
+}
 paste.collapse <- function(...)
 {
     paste(..., collapse=" ")
@@ -692,7 +764,7 @@ pastef <- function(s, fmt, ...) # paste the printf style args to s
 {
     paste0(s, sprint(fmt, ...))
 }
-print.first.few.elements.of.vector <- function(x, trace, name=NULL)
+print_first_few_elements_of_vector <- function(x, trace, name=NULL)
 {
     try(cat(" min", min(x), "max", max(x)), silent=TRUE)
     spaces <- "               "
@@ -794,9 +866,9 @@ quotify <- function(s, quote="\"") # add quotes and collapse to a single string
     if(is.null(s))
         "NULL"
     else if(length(s) == 0)
-        paste0(quote, quote)       # not sure what is best here
-    else if(substr(s[1], 1, 1) == quote) # already has quotes?
-        paste.collapse(s)
+        paste0(quote, quote)
+    else if(substr(s[1], 1, 1) == "'" || substr(s[1], 1, 1) == "\"")
+        paste.collapse(s) # already has quotes
     else
         paste0(quote, paste(s, collapse=paste0(quote, " ", quote)), quote)
 }
@@ -827,6 +899,8 @@ range1 <- function(object, ...)
         object <- object[,1]
     if(is.factor(object))
         c(1, nlevels(object))
+    else if(inherits(object, "Date")) # Sep 2020, R 4.0.2: range no longer works with Date objects
+        c(min(object), max(object))
     else
         range(object, finite=TRUE, ...)
 }
@@ -892,22 +966,24 @@ stopf <- function(fmt, ...) # args like printf
 stopifnot.string <- function(s, name=short.deparse(substitute(s)),
                              null.ok=FALSE, allow.empty=FALSE)
 {
+    if(name %in% c("NULL", "NA"))
+        name <- "argument"
     if(is.null(s)) {
         if(null.ok)
             return()
         else
-            stop0("'", name, "' is NULL (it should be a string)")
+            stop0(quotify(name, "'"), " is NULL (it should be a string)")
     }
     if(!is.character(s))
-        stop0("'", name, "' is not a character variable (class(",
+        stop0(quotify(name, "'"), " is not a character variable (class(",
               name, ") is \"", class(s), "\")")
     if(length(s) == 0)
-        stop0("'", name, "' is empty (it has no elements)")
+        stop0(quotify(name, "'"), " is empty (it has no elements)")
     if(length(s) != 1)
-        stop0("'", name, "' has more than one element\n       ",
+        stop0(quotify(name, "'"), " has more than one element\n       ",
               name, " = c(", paste.trunc("\"", s, "\"", sep=""), ")")
     if(!allow.empty && !nzchar(s))
-        stop0("'", name, "' is an empty string")
+        stop0(quotify(name, "'"), " is an empty string")
 }
 strip.deparse <- function(object) # deparse, collapse, remove most white space
 {
@@ -963,7 +1039,7 @@ text.on.white <- function(x, y, label,
     else
         stop0("srt=", srt, " is not allowed (only 0, 90, and -90 are supported)")
 }
-to.logical <- function(object, len)
+to.logical <- function(object, len) # object can be a boolean or numeric vector
 {
     xlogical <- repl(FALSE, len)
     xlogical[object] <- TRUE
@@ -1050,3 +1126,43 @@ warning0 <- function(...)
 {
     warning(..., call.=FALSE)
 }
+# Binomial pairs response: fraction true for each row.
+#
+# This function is used by both earth and plotmo.
+# If you change it here, change it there too.
+#
+# The first column of y is considered to be "true", the second "false".
+#
+# Example y:
+#   survived died
+#      1        1
+#      0        0   # both values zero
+#      3        4
+#
+# becomes:
+#   survived
+#         .5        # 1 / (1 + 1)
+#          0        # special case (both survived and died equal to 0)
+#        .43        # 3 / (3 + 4)
+
+bpairs.yfrac <- function(y, trace)
+{
+    stopifnot(NCOL(y) == 2)
+    both.zero <- (y[,1] == 0) & (y[,2] == 0)
+    y[both.zero, 2] <- 1 # so zero rows will be translated to 0 in next line
+    yfrac <- y[, 1, drop=FALSE] / (y[,1] + y[,2]) # fraction true
+    trace.bpairs.yfrac(yfrac, trace)
+    yfrac
+}
+trace.bpairs.yfrac <- function(yfrac, trace)
+{
+    # based on code in print.earth.fit.args
+    if(trace >= 4)
+        cat("\n")
+    if(trace >= 1 && trace < 7) { # don't print matrices when doing very detailed earth.c tracing
+        tracex <- if(trace >= 5) 4 else 2 # adjust trace for print_summary
+        details <- if(trace >= 4) 2 else if(trace >= 1) -1 else 0
+        print_summary(yfrac, "yfrac", tracex, details=details)
+        if(details > 1) printf("\n")
+    }
+}
